@@ -20,21 +20,30 @@
 #include "KSMainWindow.h"
 #include "Config.h"
 
+#include <QDesktopServices>
+#include <QJsonArray>
 #include <QPrintDialog>
+#include <QShortcut>
+#include <QTimer>
+#include <QPushButton>
+#include <QVBoxLayout>
+
 #ifdef XCB_FOUND
 #include <QX11Info>
 #include <xcb/xcb.h>
 #endif
 
+#include <KLocalizedString>
 #include <KGuiItem>
 #include <KStandardAction>
 #include <KStandardGuiItem>
 #include <KHelpMenu>
 #include <KAboutData>
 
-#include "KSSaveConfigDialog.h"
+#include "SettingsDialog/SettingsDialog.h"
 #include "ExportMenu.h"
 #include "ExportManager.h"
+#include "SpectacleConfig.h"
 
 KSMainWindow::KSMainWindow(bool onClickAvailable, QWidget *parent) :
     QDialog(parent),
@@ -45,7 +54,7 @@ KSMainWindow::KSMainWindow(bool onClickAvailable, QWidget *parent) :
     mClipboardButton(new QToolButton),
     mSaveButton(new QToolButton),
     mSaveMenu(new QMenu),
-    mCopyMessage(new KMessageWidget),
+    mMessageWidget(new KMessageWidget),
     mExportMenu(new ExportMenu(this)),
     mOnClickAvailable(onClickAvailable)
 {
@@ -118,7 +127,7 @@ void KSMainWindow::init()
 
     mDialogButtonBox->setStandardButtons(QDialogButtonBox::Help | QDialogButtonBox::Discard);
 
-    KGuiItem::assign(mSendToButton, KGuiItem(i18n("Export To...")));
+    KGuiItem::assign(mSendToButton, KGuiItem(i18n("Export Image...")));
     mSendToButton->setIcon(QIcon::fromTheme(QStringLiteral("application-x-executable")));
     mDialogButtonBox->addButton(mSendToButton, QDialogButtonBox::ActionRole);
 
@@ -129,20 +138,10 @@ void KSMainWindow::init()
     mClipboardButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     mDialogButtonBox->addButton(mClipboardButton, QDialogButtonBox::ActionRole);
 
-    mSaveMenu->addAction(KStandardAction::save(this, SLOT(save()), this));
-    mSaveMenu->addAction(KStandardAction::saveAs(this, SLOT(saveAs()), this));
-    mSaveMenu->addAction(KStandardAction::print(this, SLOT(showPrintDialog()), this));
-    mSaveMenu->addAction(QIcon::fromTheme(QStringLiteral("applications-system")), i18n("Configure Save Options"), this, SLOT(showSaveConfigDialog()));
-
-    QAction *saveAndExitAction = new QAction(QIcon::fromTheme(QStringLiteral("document-save")), i18n("Save &&& Exit"), this);
-    saveAndExitAction->setToolTip(i18n("Save screenshot in your Pictures directory and exit"));
-    saveAndExitAction->setShortcut(QKeySequence(QKeySequence::Quit));
-    connect(saveAndExitAction, &QAction::triggered, this, &KSMainWindow::saveAndExit);
-
-    mSaveButton->setDefaultAction(saveAndExitAction);
     mSaveButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
     mSaveButton->setMenu(mSaveMenu);
     mSaveButton->setPopupMode(QToolButton::MenuButtonPopup);
+    buildSaveMenu();
     mDialogButtonBox->addButton(mSaveButton, QDialogButtonBox::ActionRole);
 
     QShortcut *shortcut = new QShortcut(QKeySequence(Qt::Key_Escape), mDialogButtonBox->button(QDialogButtonBox::Discard));
@@ -154,11 +153,8 @@ void KSMainWindow::init()
     KHelpMenu *helpMenu = new KHelpMenu(this, KAboutData::applicationData(), true);
     mDialogButtonBox->button(QDialogButtonBox::Help)->setMenu(helpMenu->menu());
 
-    // copy-to-clipboard message
-
-    mCopyMessage->setText(i18n("The screenshot has been copied to the clipboard."));
-    mCopyMessage->setMessageType(KMessageWidget::Information);
-    mCopyMessage->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
+    // message widget
+    connect(mMessageWidget, &KMessageWidget::linkActivated, &QDesktopServices::openUrl);
 
     // layouts
 
@@ -167,14 +163,15 @@ void KSMainWindow::init()
 
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->addWidget(mKSWidget);
-    layout->addWidget(mCopyMessage);
+    layout->addWidget(mMessageWidget);
     layout->addWidget(mDivider);
     layout->addWidget(mDialogButtonBox);
-    mCopyMessage->hide();
+    mMessageWidget->hide();
 
     // populate our send-to actions
 
     mSendToButton->setMenu(mExportMenu);
+    connect(mExportMenu, &ExportMenu::imageShared, this, &KSMainWindow::showImageSharedFeedback);
 
     // disable onClick mode if not available on the platform
 
@@ -185,6 +182,50 @@ void KSMainWindow::init()
     // done with the init
 }
 
+void KSMainWindow::buildSaveMenu()
+{
+    // first clear the menu
+    mSaveMenu->clear();
+
+    // get our actions in order
+    QAction *actionSave = KStandardAction::save(this, SLOT(save()), this);
+    QAction *actionSaveAs = KStandardAction::saveAs(this, SLOT(saveAs()), this);
+    QAction *actionSaveExit = new QAction(QIcon::fromTheme(QStringLiteral("document-save")), i18n("Save &&& Exit"), this);
+    actionSaveExit->setToolTip(i18n("Save screenshot in your Pictures directory and exit"));
+    actionSaveExit->setShortcut(QKeySequence(QKeySequence::Quit));
+    connect(actionSaveExit, &QAction::triggered, this, &KSMainWindow::saveAndExit);
+
+    // static or dynamic
+    SpectacleConfig *cfgManager = SpectacleConfig::instance();
+    int switchState = cfgManager->useDynamicSaveButton() ? cfgManager->lastUsedSaveMode() : 0;
+
+    // put the actions in order
+    switch (switchState) {
+    case 1:
+        mSaveButton->setDefaultAction(actionSave);
+        mSaveMenu->addAction(actionSaveExit);
+        mSaveMenu->addAction(actionSaveAs);
+        break;
+    case 2:
+        mSaveButton->setDefaultAction(actionSaveAs);
+        mSaveMenu->addAction(actionSaveExit);
+        mSaveMenu->addAction(actionSave);
+        break;
+    case 0:
+    default:
+        mSaveButton->setDefaultAction(actionSaveExit);
+        mSaveMenu->addAction(actionSave);
+        mSaveMenu->addAction(actionSaveAs);
+        break;
+    }
+
+    // finish off building the menu
+    mSaveMenu->addAction(KStandardAction::print(this, SLOT(showPrintDialog()), this));
+    mSaveMenu->addSeparator();
+    mSaveMenu->addAction(QIcon::fromTheme(QStringLiteral("applications-system")), i18n("Preferences"),
+                         this, SLOT(showPreferencesDialog()));
+}
+
 // overrides
 
 void KSMainWindow::moveEvent(QMoveEvent *event)
@@ -209,6 +250,8 @@ void KSMainWindow::captureScreenshot(ImageGrabber::GrabMode mode, int timeout, b
 void KSMainWindow::setScreenshotAndShow(const QPixmap &pixmap)
 {
     mKSWidget->setScreenshotPixmap(pixmap);
+    mExportMenu->imageUpdated(ExportManager::instance()->pixmapDataUri());
+
     setWindowTitle(i18nc("Unsaved Screenshot", "Unsaved[*]"));
     setWindowModified(true);
 
@@ -227,17 +270,38 @@ void KSMainWindow::showPrintDialog()
     delete printer;
 }
 
+void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
+{
+    if (error) {
+        mMessageWidget->setMessageType(KMessageWidget::Error);
+        mMessageWidget->setText(i18n("There was a problem sharing the image: %1", message));
+        mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-error")));
+    } else {
+        mMessageWidget->setMessageType(KMessageWidget::Positive);
+        mMessageWidget->setText(i18n("You can find the shared image at: <a href=\"%1\">%1</a>", message));
+        mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-ok-apply")));
+    }
+
+    mMessageWidget->animatedShow();
+    QTimer::singleShot(20000, mMessageWidget, &KMessageWidget::animatedHide);
+}
+
 void KSMainWindow::sendToClipboard()
 {
     ExportManager::instance()->doCopyToClipboard();
-    mCopyMessage->animatedShow();
-    QTimer::singleShot(5000, mCopyMessage, &KMessageWidget::animatedHide);
+
+    mMessageWidget->setMessageType(KMessageWidget::Information);
+    mMessageWidget->setText(i18n("The screenshot has been copied to the clipboard."));
+    mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
+
+    mMessageWidget->animatedShow();
+    QTimer::singleShot(10000, mMessageWidget, &KMessageWidget::animatedHide);
 }
 
-void KSMainWindow::showSaveConfigDialog()
+void KSMainWindow::showPreferencesDialog()
 {
-    KSSaveConfigDialog saveDialog(this);
-    saveDialog.exec();
+    SettingsDialog prefDialog(this);
+    prefDialog.exec();
 }
 
 void KSMainWindow::setScreenshotWindowTitle(QUrl location)
@@ -249,16 +313,21 @@ void KSMainWindow::setScreenshotWindowTitle(QUrl location)
 
 void KSMainWindow::save()
 {
+    SpectacleConfig::instance()->setLastUsedSaveMode(1);
+    buildSaveMenu();
     ExportManager::instance()->doSave();
 }
 
 void KSMainWindow::saveAs()
 {
+    SpectacleConfig::instance()->setLastUsedSaveMode(2);
+    buildSaveMenu();
     ExportManager::instance()->doSaveAs(this);
 }
 
 void KSMainWindow::saveAndExit()
 {
+    SpectacleConfig::instance()->setLastUsedSaveMode(0);
     ExportManager::instance()->doSave();
     QApplication::quit();
 }
